@@ -1,18 +1,18 @@
 #' Determination of directions for improvement
-#' 
+#'
 #' determines directions for improvement
-#'  
+#'
 #' @param o_splx : matrix of objectives of simplexes (nrow = npoints, ncol = nobj)
 #' @param f_splx : vector (npoints) of associated Pareto numbers (1 = dominated)
 #' @return list of elements "oriedge": oriented edges and "ledge": length
-#' 
+#'
 #' @examples
 #' # Definition of the parameters
 #' o_splx <- matrix(rexp(6), 3, 2)
 #' f_splx <- c(1,1,1)
 #' # Call the function
 #' res <- Dimprove(o_splx, f_splx)
-#' 
+#'
 #' @author Fabrice Zaoui
 
 Dimprove <- function(o_splx, f_splx) {
@@ -22,7 +22,7 @@ Dimprove <- function(o_splx, f_splx) {
   oriedge <- NULL
   ledge <- NULL
 
-  for (k in 1:length(if1)) {
+  for (k in seq_len(length(if1))) {
     is_dominated <- dominated(o_splx[if1[k], ], o_splx)
     ideb <- which(is_dominated == TRUE)
     if (length(ideb) > 0) {
@@ -34,6 +34,6 @@ Dimprove <- function(o_splx, f_splx) {
       ledge <- rbind(ledge, lk)
     }
   }
-  
+
   return(list("oriedge" = oriedge, "ledge" = ledge))
 }
